(* -*-sml-*- *)

(* ----------------------------------------------------------
 * This file contains some test cases for the subtyping mode
 * of the interpreter.
 * The typing mode is choosen at the beginning to be simply
 * typed and subtyping is enabled.
 *
 * author: Christian Mueller
 * --------------------------------------------------------- *)

switch tm s
switch sub

(* 1. standard example: subtyping on records *)

def f = \r:{x:Int}. r@x + 3 

rdef result_1 = f {x=0,y=true} 


(* 2. subtyping on variants, no ascription needed ! *)

def g = \v:<a:Int,b:Bool>. case v of 
			       <a=x> => x			     
			     | <b=x> => (if x then 1 else 0) 

rdef result_21 = g <a=2> 
rdef result_22 = g <b=true> 
rdef result_23 = g <b=false> as <a:Int,b:Bool> 

(* 3. supremum in if expression *)

rdef result_31 = \b:Bool. if b then {a=3,b=4} else {b=5} 
rdef result_32 = \b:Bool. if b then \r:{a:Int}.1 else \r:{a:Int,b:Bool}.1 

(* 4. supremum in case expression *)

rdef result_4 = \v:<a:Int,b:Bool,c:Unit>. case v of 
                                            <a=x> => {a=x,c=unit} 
					  | <b=x> => {a=2,c=unit} 
                                          | <c=x> => {a=3,b=true} 

(* 5. subtyping for built-in fixpoint operator *)

def result_5 = fix (\f:{a:Int}->{a:Int,b:Int}.\r:{a:Int}. 
		    if r@a >> 0 then f {a=r@a-1,b=r@a-1} else {a=r@a,b=r@a}) 

(* 6. subtyping on iso-recursive types *)

def h = \f:mu X.{a:Int,b:Int}->X.1 

def R = mu X.{a:Int}->X

def i = fold R (fix (\f:{a:Int}->R.\x:{a:Int}.fold R f))

def result_6 = h i

(* 
   this is not legal for the subtyping relation, 
   check it out on a piece of paper 
*)

(*
 def omega' = \x:mu X.X->{a:Int}.(unfold (mu X.X->{a:Int}) x) x
 def result_6' = \x:mu Y.Y->{a:Int,b:Bool}. omega x
*)

(* 7. test case for cc; subkinding on type operators *)

switch tm cc 

def F = \X:{a:*}.X@a

rdef result_7 = \x:F {a=Int,b=Bool}.x

